package boshclient

import (
	"fmt"
	"net/http"
)

func (c *Client) GetTaskState(taskID int) (BoshTask, error) {
	var getTaskResponse BoshTask

	err := c.getDataFromBoshCheckingForErrors(
		fmt.Sprintf("%s/tasks/%d", c.boshURL, taskID),
		http.StatusOK,
		&getTaskResponse,
	)

	if err != nil {
		// this if block is a work around for BOSH bug #1225 - remove once fixed!
		if e, ok := err.(unexpectedStatusError); ok && e.actualStatus == http.StatusUnauthorized {
			return BoshTask{State: Done, Description: "", Result: ""}, nil
		}
		return BoshTask{}, err
	}

	return getTaskResponse, nil
}

type TaskOutput struct {
	ExitCode int    `json:"exit_code"`
	StdOut   string `json:"stdout"`
	StdErr   string `json:"stderr"`
}

func (c *Client) GetTaskOutput(taskID int) ([]TaskOutput, error) {
	outputs := []TaskOutput{}
	var output TaskOutput
	outputReadyCallback := func() {
		outputs = append(outputs, output)
		output = TaskOutput{}
		// `output` is reused for JSON decoding, so use a fresh struct;
		// else you will override your previous values with the current one
	}

	err := c.getMultipleDataFromBoshCheckingForErrors(
		fmt.Sprintf("%s/tasks/%d/output?type=result", c.boshURL, taskID),
		http.StatusOK,
		&output,
		outputReadyCallback,
	)

	return outputs, err
}
