package boshclient_test

import (
	"encoding/json"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"github.com/pivotal-cf/on-demand-service-broker/boshclient"
	. "github.com/pivotal-cf/on-demand-service-broker/mockbosh"
)

var _ = Describe("tasks", func() {
	Context("gets tasks for a deployment", func() {
		var (
			deploymentName   = "an-amazing-deployment"
			actualTasks      []boshclient.BoshTask
			actualTasksError error

			expectedTasks = []boshclient.BoshTask{
				{State: "In progress", Description: "snapshot deployment", Result: "result-1"},
				{State: "Done", Description: "snapshot deployment", Result: "result-2"},
			}
		)

		JustBeforeEach(func() {
			// director.InitializeTasksListTest(deploymentName, expectedTasks, shouldFail)
			actualTasks, actualTasksError = c.GetTasks(deploymentName)
		})

		Context("when bosh fetches the task successfully", func() {
			BeforeEach(func() {
				director.VerifyAndMock(
					Tasks(deploymentName).Returns(expectedTasks),
				)
			})

			It("returns the task state", func() {
				Expect(actualTasks).To(Equal(expectedTasks))
			})

			It("does not error", func() {
				Expect(actualTasksError).NotTo(HaveOccurred())
			})
		})

		Context("when bosh fails to fetch the task", func() {
			BeforeEach(func() {
				director.VerifyAndMock(
					Tasks(deploymentName).Fails("because reasons"),
				)
			})

			It("wraps the error", func() {
				Expect(actualTasksError).To(MatchError(ContainSubstring("expected status 200, was 500")))
			})
		})
	})

	Context("deserialization", func() {
		It("reads tasks from json", func() {
			data := []byte(`[
  {
    "id": 12986,
    "state": "done",
    "description": "snapshot deployment",
    "timestamp": 1461135602,
    "result": "snapshots of deployment 'redis-on-demand-broker-dev2' created",
    "user": "scheduler",
    "deployment": "redis-on-demand-broker-dev2"
  },
  {
    "id": 12729,
    "state": "done",
    "description": "snapshot deployment",
    "timestamp": 1461049202,
    "result": "snapshots of deployment 'redis-on-demand-broker-dev2' created",
    "user": "scheduler",
    "deployment": "redis-on-demand-broker-dev2"
  },
  {
    "id": 12427,
    "state": "done",
    "description": "snapshot deployment",
    "timestamp": 1460962800,
    "result": "snapshots of deployment 'redis-on-demand-broker-dev2' created",
    "user": "scheduler",
    "deployment": "redis-on-demand-broker-dev2"
  }]`)
			var tasks []boshclient.BoshTask
			Expect(json.Unmarshal(data, &tasks)).To(Succeed())

			Expect(tasks).To(Equal([]boshclient.BoshTask{
				{
					ID:          12986,
					State:       "done",
					Description: "snapshot deployment",
					Result:      "snapshots of deployment 'redis-on-demand-broker-dev2' created",
				},
				{
					ID:          12729,
					State:       "done",
					Description: "snapshot deployment",
					Result:      "snapshots of deployment 'redis-on-demand-broker-dev2' created",
				},
				{
					ID:          12427,
					State:       "done",
					Description: "snapshot deployment",
					Result:      "snapshots of deployment 'redis-on-demand-broker-dev2' created",
				},
			}))
		})
	})
})
