package boshclient

import (
	"fmt"
	"net/http"
	"time"

	"github.com/pivotal-cf/on-demand-service-broker-sdk/bosh"
)

func (c *Client) VMs(name string) (bosh.BoshVMs, error) {
	errs := func(err error) (bosh.BoshVMs, error) {
		return nil, err
	}

	taskID, err := c.getTaskIdFromBoshCheckingForErrors(
		fmt.Sprintf("%s/deployments/%s/vms?format=full", c.boshURL, name),
		http.StatusFound,
	)

	if err != nil {
		return errs(err)
	}

	// TODO we only test the polling logic at system test level (when real bosh queues the task, and then taskes a long time to do it)
	// We should change that
	for {
		taskState, err := c.GetTaskState(taskID)
		if err != nil {
			return errs(err)
		}

		if taskState.State == Error {
			return nil, fmt.Errorf("task %d failed", taskID)
		}

		if taskState.State == Done {
			c.logger.Printf("Task %d finished with state %v\n", taskID, taskState)
			break
		}

		time.Sleep(time.Second * c.BoshPollingInterval)
	}

	vmsOutputForEachJob, err := c.VMsOutput(taskID)
	if err != nil {
		return errs(err)
	}

	vms := bosh.BoshVMs{}
	for _, vmsOutput := range vmsOutputForEachJob {
		vms[vmsOutput.InstanceGroup] = append(vms[vmsOutput.InstanceGroup], vmsOutput.IPs...)
	}

	return vms, nil
}

type BoshVMsOutput struct {
	IPs           []string
	InstanceGroup string `json:"job_name"`
}

func (c *Client) VMsOutput(taskID int) ([]BoshVMsOutput, error) {
	outputs := []BoshVMsOutput{}
	var output BoshVMsOutput
	outputReadyCallback := func() {
		outputs = append(outputs, output)
		output = BoshVMsOutput{}
		// `output` is reused for JSON decoding, so use a fresh struct;
		// else you will override your previous values with the current one
	}

	err := c.getMultipleDataFromBoshCheckingForErrors(
		fmt.Sprintf("%s/tasks/%d/output?type=result", c.boshURL, taskID),
		http.StatusOK,
		&output,
		outputReadyCallback,
	)

	return outputs, err
}
