package broker

import (
	"fmt"

	"github.com/pivotal-cf/brokerapi"
	"github.com/pivotal-cf/on-demand-service-broker-sdk/bosh"
	"github.com/pivotal-cf/on-demand-service-broker-sdk/serviceadapter"
)

func (b *Broker) Bind(instanceID, bindingID string, details brokerapi.BindDetails) (brokerapi.Binding, error) {
	errs := func(context string, err error) (brokerapi.Binding, error) {
		b.logger.Printf("error %s: %s", context, err)
		return brokerapi.Binding{}, convertToDisplayableError(err, b.logger)
	}

	vms, manifest, err := b.gatherBindingContext(instanceID)
	if err != nil {
		return errs("gathering binding info", err)
	}

	b.logger.Printf("service adapter will create binding with ID %s for instance %s\n", bindingID, instanceID)
	adapterBinding, err := b.serviceAdapter.CreateBinding(bindingID, vms, manifest, details.Parameters)
	switch err := err.(type) {
	case serviceadapter.BindingAlreadyExistsError:
		b.logger.Printf("error: %s\n", err)
		return errs("creating binding", DisplayableError{brokerapi.ErrBindingAlreadyExists})
	case serviceadapter.AppGuidNotProvidedError:
		b.logger.Printf("error: %s\n", err)
		return errs("creating binding", DisplayableError{brokerapi.ErrAppGuidNotProvided})
	case error:
		return errs("creating binding", err)
	}

	return brokerapi.Binding{
		Credentials:     adapterBinding.Credentials,
		SyslogDrainURL:  adapterBinding.SyslogDrainURL,
		RouteServiceURL: adapterBinding.RouteServiceURL,
	}, nil
}

func (b *Broker) gatherBindingContext(instanceID string) (bosh.BoshVMs, []byte, error) {
	vms, err := b.boshClient.VMs(deploymentName(instanceID))
	if err != nil {
		return nil, nil, err
	}
	manifest, err := b.boshClient.GetManifestRaw(deploymentName(instanceID))
	if manifest == nil {
		return nil, nil, fmt.Errorf("manifest not found for deployment: %s", instanceID)
	}

	return vms, manifest, err
}
