package broker_test

import (
	"errors"

	"github.com/pivotal-cf/on-demand-service-broker/broker"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
)

var _ = Describe("Initializing the broker", func() {
	Context("when there are no pre-existing instances", func() {
		BeforeEach(func() {
			instanceRepo.AllInstancesReturns([]broker.Instance{}, nil)
		})

		It("returns no error", func() {
			Expect(brokerCreationErr).NotTo(HaveOccurred())
		})
	})

	Context("when there are no pre-existing instances of configured plans", func() {
		BeforeEach(func() {
			instanceRepo.AllInstancesReturns([]broker.Instance{
				{ID: "instance1", PlanID: existingPlanID},
			}, nil)
		})

		Context("service catalog contains, the existing instance's plan", func() {
			It("returns no error", func() {
				Expect(brokerCreationErr).NotTo(HaveOccurred())
			})
		})

		Context("service catalog does not contains, the existing instance's plan", func() {
			BeforeEach(func() {
				instanceRepo.AllInstancesReturns([]broker.Instance{
					{PlanID: "i-dont-exist-in-the-catalog"},
				}, nil)
			})

			It("returns an error", func() {
				Expect(brokerCreationErr).To(MatchError(ContainSubstring("You cannot change the plan_id of a plan that has existing service instances")))
			})
		})

		Context("when instances cannot be retrieved", func() {
			BeforeEach(func() {
				instanceRepo.AllInstancesReturns(nil, errors.New("flurp"))
			})

			It("returns an error", func() {
				Expect(brokerCreationErr).To(HaveOccurred())
			})
		})
	})
})
