package broker

import (
	"fmt"

	"github.com/pivotal-cf/brokerapi"
)

func (b *Broker) Deprovision(instanceID string, _ brokerapi.DeprovisionDetails, asyncAllowed bool) (brokerapi.IsAsync, error) {
	b.deploymentLock.Lock()
	defer b.deploymentLock.Unlock()

	if !asyncAllowed {
		return true, brokerapi.ErrAsyncRequired
	}

	errs := func(err error) (brokerapi.IsAsync, error) {
		b.logger.Println(err)
		return true, convertToDisplayableError(err, b.logger)
	}

	exists, err := b.instanceRepo.InstanceExists(instanceID)
	if err != nil {
		return errs(fmt.Errorf("error deprovisioning: checking instance existence: %s", err))
	}

	if !exists {
		b.logger.Println(fmt.Errorf("error deprovisioning: instance %s, not found ", instanceID))
		return errs(DisplayableError{brokerapi.ErrInstanceDoesNotExist})
	}

	instance, err := b.instanceRepo.FindByID(instanceID)
	if err != nil {
		return errs(fmt.Errorf("error deprovisioning: finding instance: %s", err))
	}

	b.logger.Printf("deleting deployment for instance %s\n", instanceID)
	taskID, err := b.boshClient.DeleteDeployment(deploymentName(instanceID))
	if err != nil {
		return errs(fmt.Errorf("error deprovisioning: deleting bosh deployment: %s", err))
	}

	b.logger.Printf("Bosh task id for Delete instance %s was %d\n", instanceID, taskID)

	if err := b.instanceRepo.Save(Instance{ID: instanceID, LastOperationType: Delete, PlanID: instance.PlanID}); err != nil {
		return errs(fmt.Errorf("error deprovisioning: can't save instance: %s", err))
	}

	return true, nil
}
