package broker

import "github.com/pivotal-cf/brokerapi"

func (b *Broker) Update(instanceID string, details brokerapi.UpdateDetails, asyncAllowed bool) (brokerapi.IsAsync, error) {
	b.deploymentLock.Lock()
	defer b.deploymentLock.Unlock()

	errs := func(err error) (brokerapi.IsAsync, error) {
		return true, convertToDisplayableError(err, b.logger)
	}

	instance, err := b.instanceRepo.FindByID(instanceID)
	if err != nil {
		return errs(err)
	}

	b.logger.Printf("updating instance %s", instance.ID)
	detailsMap, err := convertToMap(details)
	if err != nil {
		return errs(err)
	}

	err = b.redeploy(instance, details.PlanID, Update, detailsMap, &details.PreviousValues.PlanID)
	return errs(err)
}
